package mccity.heroes.skills.turret;

import com.herocraftonline.heroes.characters.Hero;
import com.herocraftonline.heroes.characters.skill.SkillConfigManager;
import com.herocraftonline.heroes.util.Messaging;
import org.bukkit.block.Block;
import org.bukkit.entity.Player;

import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;

public class TurretManager implements Runnable {

    private final TurretMap turretMap;

    private final SkillTurret skill;
    private final TurretStorage turretStorage;

    public TurretManager(SkillTurret skill) {
        this.skill = skill;
        turretStorage = new TurretStorage(skill.plugin.getDataFolder());
        turretMap = new TurretMap();
    }

    private boolean isReplaceOldestOnLimitFor(Hero hero) {
        return SkillConfigManager.getUseSetting(hero, skill, "replace-oldest-on-limit", false);
    }

    public boolean canAddFor(Hero hero) {
        List<Turret> playerTurrets = turretMap.getByPlayer(hero.getPlayer().getName());
        return (isReplaceOldestOnLimitFor(hero) || playerTurrets.size() < skill.maxTurretsFor(hero));
    }

    public int getTotalTurrets(Hero hero) {
        return turretMap.getByPlayer(hero.getPlayer().getName()).size();
    }

    public boolean addFor(Hero hero, Block base) {
        List<Turret> playerTurrets = turretMap.getByPlayer(hero.getPlayer().getName());

        boolean replace = false;
        if (playerTurrets.size() >= skill.maxTurretsFor(hero)) {
            if (isReplaceOldestOnLimitFor(hero)) {
                // remove oldest
                turretMap.removeByBase(playerTurrets.get(0).getBaseBlock());
                replace = true;
            } else {
                return false;
            }
        }

        turretMap.add(new Turret(hero, base, skill));
        if (replace) {
            Messaging.send(hero.getPlayer(), Messages.turretReplaced);
        }
        return true;
    }

    public void onDispenserDestroyed(Block dispenser, Player destroyer) {
        Turret destroyed = turretMap.removeByBase(dispenser);
        if (destroyed != null) {
            if (destroyer.getName().equals(destroyed.getOwnerName())) {
                Messaging.send(destroyer, Messages.yourTurretDestroyed);
            } else {
                Messaging.send(destroyer, Messages.youDestroyOtherPlayerTurret, destroyed.getOwnerName());
                Utils.safeSend(destroyed.getOwnerName(), Messages.yourTurretDestroyedBy, destroyer.getName());
            }
        }
    }

    public void removeAll(Hero hero) {
        turretMap.removeByPlayer(hero.getPlayer().getName());
    }

    @Override
    public void run() {
        List<Turret> turrets = turretMap.getContent();
        List<Turret> forRemove = new ArrayList<Turret>();

        for (Turret curTurret : turrets) {
            if (curTurret.isAlive()) {
                if (curTurret.getBaseBlock().getChunk().isLoaded()) {
                    curTurret.onAiTick();
                }
            } else {
                forRemove.add(curTurret);
            }
        }

        for (Turret rTurret : forRemove) {
            turretMap.removeByBase(rTurret.getBaseBlock());
            Utils.safeSend(rTurret.getOwnerName(), Messages.yourTurretLostMagicPower);
        }
    }

    public void onPlayerJoin(Player player) {
        List<Turret> playerTurrets = turretMap.getByPlayer(player.getName());
        if (!playerTurrets.isEmpty()) {
            Hero owner = skill.plugin.getCharacterManager().getHero(player);
            for (Turret playerTurret : playerTurrets) {
                playerTurret.ownerJoined(owner);
            }
        }
    }

    public void onPlayerQuit(Player player) {
        List<Turret> playerTurrets = turretMap.getByPlayer(player.getName());
        for (Turret playerTurret : playerTurrets) {
            playerTurret.ownerQuited();
        }
    }

    public void loadData() {
        List<Turret> loaded = turretStorage.load();
        Utils.log(loaded.size() + " turrets loaded", Level.INFO);
        turretMap.setContent(loaded);
    }

    public void saveData() {
        turretStorage.save(turretMap.getContent());
    }
}
